from pdfriend.commands.version import version
from pdfriend.commands.merge import merge
from pdfriend.commands.invert import invert
from pdfriend.commands.edit import edit
from pdfriend.commands.tinker import tinker
from pdfriend.commands.cache import cache
from pdfriend.commands.weave import weave
from pdfriend.commands.split import split
from pdfriend.commands.encrypt import encrypt
from pdfriend.commands.decrypt import decrypt
from pdfriend.commands.metadata import metadata
from pdfriend.commands.get import get
